<?php

class DPSHostedPayment extends Payment {
	
	// DPS account information
	protected static $dps_user_id; # Px Pay user id (set in yoursitename/_config.php)
	protected static $dps_key; # Px Pay key (set in mysite/_config)
	
	// Relevant DPS URLs
	protected static $dps_url; # URL to post transaction data to (set in mysite/_config)
	protected static $logo = 'http://www.paymentexpress.com/images/dps_paymentexpress.png'; # dps|paymentexpress logo
	protected static $privacy = 'http://www.paymentexpress.com/privacypolicy.htm'; # link to dps' privacy policy
	
	// Default Transaction type
	protected static $dps_transaction_type;
	
	// Function to set the account credentials (set in yoursitename/_config.php)
	static function set_account($user_id, $key, $url) {
		self::$dps_user_id = $user_id;
		self::$dps_key = $key;
		self::$dps_url = $url;
	}
	
	// Function to control default transcation (set in yoursitename/_config.php)
	static function set_transaction_type ($type) {
		self::$dps_transaction_type = $type;
	}
	
	// Function to post xml (via cURL) to DPS
	// Returns xml output or false on failure
	// TODO: implement !function_exists(curl_init) check to determine if cURL is installed
	static function post_to_dps($xml) {
	
		$ch = curl_init(); 
		curl_setopt($ch, CURLOPT_URL, self::$dps_url);
		
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS,$xml);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		// set up proxy, this may change depending on ISP, please contact your ISP to get the correct cURL settings
		# curl_setopt($ch,CURLOPT_PROXY , "proxy:8080");
		# curl_setopt($ch,CURLOPT_PROXYUSERPWD,"username:password");
		$xml_output = curl_exec ($ch); 		
		// Debug:
		# print("<pre>output_xml: " . print_r($xml_output, true) . "</pre>");
		
		curl_close ($ch);
		return ($xml_output) ? $xml_output : false;
	}
	
	function getPaymentFormFields() {
		return new FieldSet(
			new LiteralField("RedirectNotice", "<p>You will be redirected to DPS' secure website to enter your payment details.</p><br />"),
			new LiteralField("DPSInfo", '<a href="' . self::$privacy . '" title="Read DPS\' privacy policy"><img src="' . self::$logo . '" alt="Payments powered by DPS" /></a>')
		);	
	}
	
	function getPaymentFormRequirements() {
		return null;
	}
	
	function processPayment($data, $form) {
		$page = new Page();

		$page->Title = 'Redirection to DPS';
		$page->Logo = '<img src="' . self::$logo . '" alt="Payments powered by DPS"/>';
		$page->Form = $this->DPSForm();
		
		$controller = new Page_Controller($page);
		
		Requirements::javascript(THIRDPARTY_DIR . '/jquery/jquery.js');
		
		$form = $controller->renderWith('PaymentProcessingPage');
		
		return new Payment_Processing($form);
	}

	function DpsForm() {
				
		$order = $this->Order();
		$member = $order->Member();
		
		// Prepare values to populate XML fields for DPS
		$return_url = Director::absoluteBaseURL(). 'DPSHostedPayment_Handler/complete/';
		// $txntype =  $this->ManuallyConfirm ? "Auth" : "Purchase"; # saw this in another module - not sure what it does?
		$amount_input = number_format($this->Amount, 2); # ensure the amount returns as d.cc format (5.00 as opposed to 5)
		$currency = $this->Currency;
		$txn_id = $this->OrderID;
		
		$xml_input = "<GenerateRequest>";
		$xml_input .= "<PxPayUserId>" . self::$dps_user_id . "</PxPayUserId>";
		$xml_input .= "<PxPayKey>" . self::$dps_key . "</PxPayKey>";
		$xml_input .= "<AmountInput>" . $amount_input . "</AmountInput>"; # $amount = $order->Total(); || $this->Amount
		$xml_input .= "<CurrencyInput>" . $currency . "</CurrencyInput>";
		$xml_input .= "<TxnType>" . self::$dps_transaction_type . "</TxnType>";
		$xml_input .= "<MerchantReference>" . "Order " . $txn_id . "</MerchantReference>";
		$xml_input .= "<TxnId>" . $txn_id . "</TxnId>";
		$xml_input .= "<UrlSuccess>" . $return_url . "</UrlSuccess>";
		$xml_input .= "<UrlFail>" . $return_url . "</UrlFail>";
		# $xml_input .= "<>" .  . "</>";
		$xml_input .= "</GenerateRequest>";
		
		// Debug:
		# print("<pre>xml_input: " . print_r(htmlentities($xml_input), true) . "</pre>");
		# exit;
		
		// Post the transaction XML data to DPS to get the hosted payment page URL to redirect to
		$xml_output = self::post_to_dps($xml_input);
		if (!$xml_output) {
			user_error('There was an error connecting to the payment gateway.', E_USER_ERROR);
		}
		else {		
			// Get URI from <Request>
			// TODO: report if valid attribute != 1 in <Request valid="x">
			// For now we (badly) assume that requests are formatted properly.
			$xmlobj = simplexml_load_string($xml_output);
			$URI = $xmlobj->URI; # the address to redirect the user to for payment.
			$URI = Convert::raw2xml($URI);
		
		return <<<HTML
			<script type="text/javascript">
				jQuery(document).ready(function() {
					location = "$URI";
				});
			</script>
HTML;

		}
	}
}

/**
 * Handler for responses from the DPS site
 * =======================================
 * The user will be redirected here after successful/failed payment attempt at the hosted page
 * URL example: http://yourdomain.com/DPSHostedPayment_Handler/complete/?result=reallyLongString&userid=yourDpsUsername;
 */
class DPSHostedPayment_Handler extends Controller {

	// DPS Information
	protected static $dps_user_id;
	protected static $dps_key;
	protected static $dps_url;

	static function set_account($user_id, $key, $url) {
		self::$dps_user_id = $user_id;
		self::$dps_key = $key;
		self::$dps_url = $url;
	}
	
	function complete() {
		if(isset($_REQUEST['result'])) {
			
			// build the xml for DPS to decrypt the URL in the result parameter
			$xml_input = "<ProcessResponse>";
			$xml_input .= "<PxPayUserId>" . self::$dps_user_id . "</PxPayUserId>";
			$xml_input .= "<PxPayKey>" . self::$dps_key . "</PxPayKey>";
			$xml_input .= "<Response>" . $_REQUEST['result'] . "</Response>";
			$xml_input .= "</ProcessResponse>";
			
			// Post the XML data to DPS to get the transaction outcome XML
			$xml_output = self::post_to_dps($xml_input);
			if (!$xml_output) {
			user_error('There was an error connecting to the payment gateway.', E_USER_ERROR);
			}
			else {
				// Get the useful values from the Transaction outcome XML
				$xmlobj = simplexml_load_string($xml_output);
				$success = $xmlobj->Success; # Success = 1, Failure = 0
				$message = $xmlobj->ResponseText; # Message regarding the outcome of the transaction
				# $dpstxnref = $xmlobj->DpsTxnRef;
				$order_id = $xmlobj->TxnId; # The SS order ID initially sent to DPS being returned
				$order_id = intval($order_id);
				
				if ($success) {
					if($payment = DataObject::get_by_id('DPSHostedPayment', $order_id)) {
						$payment->Status = $success == '1' ? 'Success' : 'Failure';
						# if($dpstxnref) $payment->TxnRef = $dpstxnref;
						if($message) $payment->Message = "$message";
					
					$payment->write();
						
					$payment->redirectToOrder();
					}
				}
				else {
					user_error('There was a problem retrieving the transaction outcome from the payment gateway.', E_USER_ERROR);
					exit;
				}
			}
		}
		else {
			user_error('No transaction information to decrypt', E_USER_ERROR);
			exit;
		}
	}
	
	// Function to post xml (via cURL) to DPS
	// Returns xml output or false on failure
	// TODO: implement !function_exists(curl_init) check to determine if cURL is installed
	static function post_to_dps($xml) {
	
		$ch = curl_init(); 
		curl_setopt($ch, CURLOPT_URL, self::$dps_url);
		
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS,$xml);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		// set up proxy, this may change depending on ISP, please contact your ISP to get the correct cURL settings
		#curl_setopt($ch,CURLOPT_PROXY , "proxy:8080");
		#curl_setopt($ch,CURLOPT_PROXYUSERPWD,"username:password");
		$xml_output = curl_exec ($ch); 		
		// Debug
		# print("<pre>output_xml: " . print_r($xml_output, true) . "</pre>");
		
		curl_close ($ch);
		return ($xml_output) ? $xml_output : false;
	}
}
	  

?>